# Qed Quick Dictionary 
# -*- coding: ISO-8859-2 -*-
# (c) Nmeth Lszl <nemethl@gyorsposta.hu>
# License: GNU GPL 2, see COPYING

from string import *
import re

class Qed:
    """ Quick Dictionary """

    split_patterns = [
        [ '[' + punctuation + whitespace + ']+', ' ' ], # replace long whitespaces and punctuation
        [ '^ ?', '' ], # trim whitespace of left side of string
        [ ' ?$', '' ], # trim whitespace of right side of string
        [ r'^\w? ', '' ], # erase 1 letter word from left
        [ r' \w?$', '' ]] # erase 1 letter word from right

    MAXWORDS = 10

    # Create our objects
    def __init__(self, index_file, dict_file, suffix):
        self.dict_file = dict_file
        self.suffix = suffix

        self.index = {}
        # read all lines of index file
        f = open(index_file,'r')
        lines = f.readlines()

        # build index hash
        for j in lines:
            pieces = split(j, None, 1)
            self.index[pieces[0]] = pieces[1]
        f.close()

    # trim, lowercase
    def trim_lowercase(self,word):
        for i in self.split_patterns:
            r = re.compile(i[0])
            if r.search(word):
                word = r.sub(i[1],word)
        return word.lower()

    # return the possible stems of the given word
    def get_stems(self, word):
        stems = [ word ]
        for i in self.suffix:
            r = re.compile(i[0])
            if r.search(word):
                stems = stems + [ r.sub(i[1],word) ]
        return stems

    # return the dictionary index of the given word
    def get_index(self, word):
        word = word + "____"
        for i in range(4,1,-1):
            if self.index.has_key(word[:i]):
                return word[:i]
        return None

    # create an index dictionary with stems
    def sort_stems_by_index(self, stems):
        sorted = {}
        for i in stems:
            j = self.get_index(i)
            if j:
                if sorted.has_key(j):
                    sorted[j] = sorted[j] + [ i ]
                else:
                    sorted[j] = [ i ]
        return sorted

    # get an item from the dictionary by word
    def look_dictionary(self, word, word2 = ""):
        item = [] # return value
        item2 = [] # return value
        
        word = self.trim_lowercase(word)

        if word == "":
            return []

        f = open(self.dict_file,'r')

        words = self.get_stems(word)

        stems_dic = {}
        stems_dic = self.sort_stems_by_index(words)

        for i in stems_dic.keys():
            place = split(self.index[i])
            f.seek(int(place[0])) # go to the first byte of the record
            lines = f.readlines(int(place[1])) # read all lines of the record

            # get dictionary items by filter record with a compiled regular exp.

            keyword = join(stems_dic[i],'|')

            filter1 = re.compile("""
                    (^to[ ] | ^ | ,[ ])                 # match verb, noun, or irregular word
                    
                    (""" + keyword + r""")
                    [ ,]                                # with a next space or comma
                    [ []                                # and a next space or bracket
                    """, re.VERBOSE | re.IGNORECASE)

            if word2 == "":
                for i in lines:
                    if filter1.search(i):
                        item = item + [i]
            elif word2 == " ":                       # all items
                filter2 = re.compile(r"""
                    \b(""" + keyword + r""")\b.*[ ][ ]  # match verb, noun, or irregular word
                    """, re.VERBOSE | re.IGNORECASE)
                for i in lines:
                    if filter2.search(i) and not filter1.search(i):
                        item = item + [i]
            else:
                filter2 = re.compile(r"""
                    \b(""" + keyword + r""")\b.*[ ][ ]  # match verb, noun, or irregular word
                    """, re.VERBOSE | re.IGNORECASE)

                filter3 = re.compile(r"""
                    \b(""" + word2 + r""")\b.*[ ][ ]    # match verb, noun, or irregular word
                    """, re.VERBOSE | re.IGNORECASE)

                for i in lines:
                    if filter2.search(i) and filter3.search(i) and not filter1.search(i):
                        item = item + [i]
        
        f.close()

        # sort items
        filter3 = re.compile("""
                    (^to[ ] | ^ | ,[ ])                # match verb, noun, or irregular word
                    (""" + word + r"|" + word2 + r""")
                    [ ,]                               # with a next space or comma
                    [ []                               # and a next space or bracket
                    """, re.VERBOSE | re.IGNORECASE)

        for i in item:
            if filter3.search(i):
                item2 = [i] + item2
            else:
                item2 = item2 + [i]

        return item2

    def get_item(self, word):
        return self.look_dictionary(word);

    def get_all(self, word):
        return self.look_dictionary(word, " ");

    def get_idiom(self, word, word2):
        if word2 != "":
            w = []
            w = self.look_dictionary(word, word2)
            if len(w) == 0:
                return self.look_dictionary(word2, word)
            else:
                return w
        else:
            return []
