# Quick English Dictionary for X
# -*- coding: ISO-8859-2 -*-

# (c) Nmeth Lszl <nemethl@gyorsposta.hu>
# License: GNU GPL 2, see COPYING

"""XQED gyorsfordt 1.0.3 (c) Nmeth Lszl, 2000, 2001, 2003, 2004
Licenc: GNU GPL 2
A Qed fordt honlapja: http://www.math.u-szeged.hu/~bognarv/qed/qed.html

A QED gyorsfordt X Window vltozata az UHU-Linux Kft. programfejlesztsi plyzatnak tmogatsval kszlt 2003-ban.
Az UHU-Linux honlapja: http://www.uhulinux.hu

Felhasznlt anyagok:
    Vony Attila: Angol-magyar sztr (http://www.mek.oszk.hu)
    Szy Gyrgy: Kiskapu sztr (http://szotar.kiskapu.hu)
    Carnegie Mellon Pronouncing Dictionary (http://www.festival.org)

"""

from Tkinter import *
from Qed import *
from string import *
import re
import os
import os.path
from time import sleep

class Xqed:
    """ Quick English Dictionary for X """

    VERSION = "1.0.3"
    maxword = 20 # max. translated words in a selection
    mode = 0 # 0 = foreground, 1 = background, 2 = command line
    sel_mode = 0 # 0 = not clear selection, 1 = clear sel. allows repeated selection
    
    prev_prono = 1
    mpos = 30 # window x and y distance from mouse pointer
    qed_prono = '/usr/share/qed/pronouncing' # XXX fixed path
    qed_use = '/usr/share/qed/usage' # XXX fixed path
    festival = '/usr/bin' # XXX fixed path
    empty_message = "A keresett sz nincs a sztrban.\n"
    empty_message2 = """Jellje ki a fordtani kvnt szt, vagy szavakat!

Ha belekattint ebbe az ablakba, akkor a fordtand szveget kzvetlenl is \
megadhatja. Fordtshoz a begpelt sor vgn nyomja meg a \
,,Shift-Home'' billentykombincit.\n\n"""
    empty_message3 = "Nincsenek kifejezsek a kijellt szval.\n"
    qed_dict = '/usr/share/qed/eng-hu.dic' # XXX fixed path
    qed_index = '/usr/share/qed/eng-hu.index' # XXX fixed path
    PIC_LOGO = "/usr/share/qed/xqed.gif"
    PIC_BG = "/usr/share/qed/xqed-bg.gif"
    PIC_LOCAL = "/usr/share/qed/xqed-local.gif"
    PIC_WINDOW = "/usr/share/qed/xqed-window.gif"
    PIC_WINDOW = "/usr/share/qed/xqed-window.gif"
    PIC_IDIOM = "/usr/share/qed/xqed-idiom.gif"
    PIC_ERROR = "A kp nem elrhet.\n"

    local_on = 0
    prev_selection = ''
    options_c = 0
    options_m = 0

    def __init__(self):
        suffix = []

        suffix = [
            [ '(ive|ion|ions|ing|ings|est|er|ers)$', 'e' ], 
            [ '(ive|en|th|ly|ing|d|est|er|s)$', '' ], 
            [ '(ens|ness|y)$', '' ],
            [ '(ings|ed|ers|es)$', ''], 
            [ r'(?<=(.))\1(ing|ings|ed|y)$', ''], # running -> run
            [ 'ck(ing|ings|ed)$', 'c'],
            [ '(ication|ied|iest|ier|iness)$', 'y'],
            [ '(ications|ies|iers|ieth)$', 'y'],
            [ 'ves$', 'f'], 
            [ 'ves$', 'fe' ]]
        
        self.dictionary = Qed(self.qed_index, self.qed_dict, suffix)

        # command line mode: translate parameters
        commandline = []
        options_a = 0
        options_b = 0
        
        if os.path.basename(sys.argv[0]) == "angolul":
            self.options_m = 1
            
        if os.getenv('XQED_COLOR'):
            self.options_c = 1
        
        if (len(sys.argv) > 1):
            for i in range(1,len(sys.argv)):
                if sys.argv[i][0] == '-':
                    if sys.argv[i] == "-b":
                        options_b = 1
                        break
                    elif sys.argv[i] == "-h":
                        print """XQED angol-magyar gyorsfordt (c) Nmeth Lszl, 2000, 2001, 2003, 2004
Hasznlat:
    xqed [-h | -b | -a | -m | -q] [-c] [szavak]
    magyarul [-h | -b | -a | -m | -q] [-c] [szavak]
    angolul [-h | -b | -a | -m | -q] [-c] [szavak]
Lers:
    Paramter nlkl, vagy a -b kapcsolval indtva a program grafikus
    fellete indul el, ami beptett lerssal rendelkezik.
    Ha (angol) szavakat adunk meg paramterknt, ezeket a program
    alaprtelmezsknt magyarra fordtja.
    Az -m, -q kapcsolkkal, vagy az ,,angolul'' nven
    indtva magyar szavak angol jelentst kereshetjk meg a sztrban.
    
Kapcsolk:
    -a  az sszes kifejezs megjelentse
    -b  a grafikus fellet httr-zemmdban indul
    -c  sznes megjelents a terminlban (less -r, ha lapozni akarjuk)
        Ha mindig sznes megjelentst szeretnnk, az XQED_COLOR
        krnyezeti vltozt lltsuk be (l. man Xqed).
    -h  lers
    -m  A megadott magyar szra keres r (nem kifejezsek rszeknt).
        Tbb paramter esetn viszont azokat a sorokat vlasztja ki,
        amelyek a paramterknt megadott szavakat, vagy sztredkeket
        (tetszleges sorrendben) mind tartalmazzk (szrs S
        kapcsolattal megadott karakterlncokra). Ez utbbi keress
        angol szavak s kifejezsek keressre is hasznlhat.
        Az ,,angolul'' parancs esetn ez az alaprtelmezett mkds.
    -p  kiejtsi informcik megjelentse
    -s  vglap trlse: a kzvetlenl megismtelt kijellseket is
        szrevesz az XQED program program, de a vglap legfeljebb
        csak az alkalmazson bell hasznlhat.
    -v  XQED vltozat szma
Pldk:
    angolul hajat vg # a kt szrszletet tartalmaz bejegyzsek kirsa
    angolul macska    # macska szval kezdd magyar jelents sztri
                        bejegyzsek megjelentse (kb. magyar-angol sztr)
    xqed              # grafikus fellet indtsa eltr-zemmdban
    angolul           # grafikus fellet indtsa eltr-zemmdban
    magyarul          # grafikus fellet indtsa eltr-zemmdban
    xqed -b &         # httr-zemmdban s httrben indts X alatt
    magyarul hurry up # a kt sz, s az ezekkel alkotott kifejezsek 
                        magyar jelentsnek megjelentse a terminlban
    magyarul mouse -a # az sszes mouse tartalm kifejezs megjelentse
    angolul we | less # elre-htra lapozhat megjelents a less programmal
    angolul -c we | less -r  # az elz sznesben
    """
                    elif sys.argv[i] == "-a":
                        options_a = 1
                    elif sys.argv[i] == "-c":
                        self.options_c = 1
                    elif sys.argv[i] == "-m":
                        self.options_m = 1
                    elif sys.argv[i] == "-v":
                        print "XQED", self.VERSION
                    elif sys.argv[i] == "-p":
                        self.mode = 2
                        self.about_prono()
                    elif sys.argv[i] == "-f":
                        self.mode = 2
                        while 1:
                            line = sys.stdin.readline()
                            if not line: break
                            self.puts([line], 'item')
                        sys.exit(0)
                    elif sys.argv[i] == "-s":
                        self.sel_mode = 1
                        break
                    else:
                        sys.stderr.write("Nincs ilyen kapcsol: " + sys.argv[i] + '\n')
                        sys.exit(1)
                else:
                    self.mode = 2
                    commandline = commandline + [sys.argv[i]]
            if len(commandline)>0:
                if options_a:
                    for i in commandline:
                        self.prev_selection = i
                        self.all_items()
                        print
#                elif options_m:
#                    self.magyar(join(commandline),"m")
                elif self.options_m:
                    print "-------------------------------------------"
                    if (len(commandline) == 1):
                        self.magyar(join(commandline),"q")
                    else:
                        self.magyar(join(commandline, "/&&/"),"q2")
                else:
                    self.search_words(join(commandline))
            if not (options_b or self.sel_mode):
                sys.exit(0)

        # interactive mode

        # main window        
        self.tk = tk = Tk()
        tk.title("XQED gyorsfordt")
        tk.iconmask('@' + '/usr/share/qed/xqed.xbm')
        tk.option_add('*font', ('helvetica', 12))
        tk.geometry("500x333+150+150")
        tk.overrideredirect(self.mode)
        
        # create menus
        self.menu = menu = Menu(tk)
        tk.config(menu=menu)
        self.fontmenu = fontmenu = Menu(menu, tearoff=0)
        fontmenu.add_command(label="8", command=self.set_font8)
        fontmenu.add_command(label="10", command=self.set_font10)
        fontmenu.add_command(label="12", command=self.set_font12)
        fontmenu.add_command(label="14", command=self.set_font14)
        fontmenu.add_command(label="18", command=self.set_font18)
        fontmenu.add_command(label="24", command=self.set_font24)
        self.modemenu = modemenu = Menu(menu, tearoff=0)
        menu.add_cascade(label="Fjl", menu=modemenu, underline=0)
        modemenu.add_command(label="sszes kifejezs", command=self.all_items, underline=9)
        modemenu.add_command(label="Felolvass", command=self.tts, underline=3)
        modemenu.add_separator()
        modemenu.add_command(label="Httr-zemmd", command=self.change_mode, underline=0)
        modemenu.add_command(label="Kilps", command=self.xqed_exit, underline=0)
        self.helpmenu = helpmenu = Menu(menu, tearoff=0)
        menu.add_cascade(label="Segtsg", menu=helpmenu, underline=0)
        helpmenu.add_command(label="Hasznlat", command=self.about_program, underline=0)
        helpmenu.add_command(label="Kiejtsi tmutat", command=self.about_prono, underline=0)
        helpmenu.add_cascade(label="Betmret".decode("iso-8859-2"), menu=fontmenu, underline=0)
        # helpmenu.add_command(label="Betmret", command=self.set_fontsize)
        helpmenu.add_command(label="Magyarrl angolra", command=self.about_magyar, underline=0)
        helpmenu.add_command(label="Az XQED nvjegye", command=self.about_xqed, underline=0)
        self.localmenu = localmenu = Menu(menu, tearoff=0, postcommand=self.set_local_on)
        localmenu.add_command(label="sszes kifejezs", command=self.all_items)
        localmenu.add_command(label="Felolvass", command=self.tts)
        localmenu.add_command(label="Kiejtsi tmutat", command=self.about_prono)
        localmenu.add_separator()
        localmenu.add_command(label="Httr-zemmd", command=self.change_mode)
        localmenu.add_command(label="Kilps", command=self.xqed_exit)

        # create frame, text widget, scrollbar
        self.frame = f = Frame(tk, bg="black")
        f.pack()
        f.bind('<Any-Leave>',self.leave)
        self.scrollbar = s = Scrollbar(f)
        s.pack(side=RIGHT,fill=Y)
        self.text = t = Text(f, wrap="word", relief=SOLID, bg="white",
                borderwidth=0, height=100, width=200, yscrollcommand=s.set)
        t.pack(side=LEFT,fill=BOTH)
        s.config(command=t.yview)        
        s.configure(width=0,borderwidth=0,elementborderwidth=0,highlightthickness=0)

        # bind right button to text widget for handling local menu
        t.bind('<Button-3>',self.local_menu)

        t.bind('<Return>',self.press_enter)

        # tags in text window for different color
        t.tag_configure('item', foreground="blue", font=('helvetica', 12, 'bold'))
        t.tag_configure('item2', foreground="black", background="gray", font=('helvetica', 14, 'bold'))
        t.tag_configure('prono', foreground="dark green", font=('helvetica', 12, 'bold'))                
        t.tag_configure('idiom', foreground="red", font=('helvetica', 12, 'bold'))                

        # start picture        
        self.pic = PhotoImage(file=self.PIC_LOGO)
        self.text.image_create(1.0,image=self.pic);

        # no initial window
        try:
            self.prev_selection = self.tk.selection_get()
        except:
            self.prev_selection = ""

        # callback
        self.tk.after(1,self.watch_selection)
        self.tk.update()

        # start in background mode
        if options_b:
            sleep(0.5)
            self.change_mode()
        
    # convert pronounciation string to more readable for Hungarians
    def hungarian_phonetic_conversion(self,text):
        text = translate(text,maketrans('aAVie&0Ou3@TwI','eaFui'))
        text = re.sub(re.compile('i'),'j',text)
        text = re.sub(re.compile('i'),'j',text)
        text = re.sub(re.compile('oi'),'oj',text)
        text = re.sub(re.compile(''),'e',text)
        text = re.sub(re.compile('s'),'sz',text)
        text = re.sub(re.compile('tS'),'cs',text)
        text = re.sub(re.compile('dZ'),'dzs',text)
        text = re.sub(re.compile('Z'),'zs',text)
        text = re.sub(re.compile('S'),'s',text)
        text = re.sub(re.compile('nj'),'ny',text)
        return text

    # selected text is a word? (For example a URL's is not)
    def is_word(self,word):
        for c in range(len(word)):
            if not (("a"<=word[c]<="z") or ("A"<=word[c]>=word[c]<="Z") or 
                    (word[c] in '-,;.?! ":()[]0123456789'+"'")):
                return 0
        return 1

    # examine content of clipboard
    def watch_selection(self):
        try:
            i = self.tk.selection_get()
            if (i <> self.prev_selection) and (self.is_word(i)==1):
                self.prev_prono = 0

                # show window, when self.mode = 1
                if self.mode:
                    t = self.tk.winfo_pointerxy()
                    self.tk.geometry("100x75+%d+%d" % \
                        (min(self.tk.winfo_screenwidth()-100,max(0,t[0]-self.mpos)), \
                        min(self.tk.winfo_screenheight()-75,max(0,t[1]-self.mpos))))
                
                self.search_words(i)

                self.set_windowsize()
                self.tk.deiconify()

                if self.sel_mode:
                    self.tk.selection_clear()
                    self.tk.clipboard_clear()
                    self.tk.clipboard_append(i)
                    self.tk.selection_get()

            self.prev_selection = i
        except:
            if not self.is_word(self.prev_selection):
                self.prev_selection = "" # unnecessary?
        # callback
        self.tk.after(1,self.watch_selection)

    # search words and idioms in dictionary
    def search_words(self, i):
        words = split(self.dictionary.trim_lowercase(i))
        empty = 0
        self.clear()
        empty = 1
        for j in range(min(len(words), self.maxword)):
            dictext = self.dictionary.get_item(words[j])
            if len(dictext) > 0:
                empty = 0
                self.puts([words[j] + '\n'],'item2')
	    else:
	        # split word	
	        for k in range(3, len(words[j]) - 2):
		    i1 = self.dictionary.get_item(words[j][0:k])
                    if len(i1)>0:
		        dictext = self.dictionary.get_item(words[j][k:])
                        if len(dictext)>0:
                            empty = 0
                            self.puts([words[j] + '\n'],'item2')
                            self.puts(i1, 'item')
            self.puts(dictext, 'item')

        if len(words) > 1:
            dictext = self.dictionary.get_idiom(words[0], words[-1])
            if len(dictext)>0:
                self.puts([words[0] + " - " + words[-1] + '\n'],'item2')
        else:
            dictext = self.dictionary.get_idiom(i,self.prev_selection)
            if len(dictext)>0:
                self.puts([words[j] + " - " +
                    self.dictionary.trim_lowercase(self.prev_selection) + '\n'],'item2')

        if empty == 1 and len(dictext) == 0:
            dictext = [ self.empty_message ]

        self.puts(dictext, 'idiom');

    def clear(self):
        if self.mode != 2:
            self.text.delete(1.0,END)

    def puts(self, dictext, color):
                for j in dictext:
                    k = split(j, '  ', 1)
                    if len(k)>1:
                        kpro = re.split('[\[\]]', k[0])
                        for ki in range(0, len(kpro)):
                            if ki % 2 == 0: # if not odd
                                self.puts_tag(kpro[ki], color)
                            else:
                                self.puts_tag(self.hungarian_phonetic_conversion('[' + 
                                    kpro[ki] + ']'), 'prono')
                        self.puts_tag('  ' + k[1], '')
                    else:
                        self.puts_tag(k[0], color)

    def puts_tag(self, text, color):
        if self.mode == 2:
	    if self.options_c:
            	if color=='item':
                    sys.stdout.write("[1m")
            	if color=='item2':
                    sys.stdout.write("[1m[7m")
            	if color=='prono':
                    sys.stdout.write("[1m[32m")
                elif color=='idiom':
                    sys.stdout.write("[1m[31m")
                sys.stdout.write(text)
                sys.stdout.write("[0m")
    	    else:
                sys.stdout.write(text)
        else:
            self.text.insert(END, text.decode("iso-8859-2"), color)

    def set_windowsize(self):
                if self.mode:
                    self.text.update()
                    winsize = 100.0;
                    while ((self.text.yview())[1]<1) and (winsize<500.0):
                        winsize *= 1.2
                        t = self.tk.winfo_pointerxy()
                        self.tk.geometry("%dx%d+%d+%d" % \
                            (winsize, winsize*0.75, \
                            min(self.tk.winfo_screenwidth()-winsize,max(0,t[0]-self.mpos)), \
                            min(self.tk.winfo_screenheight()-winsize*0.75,max(0,t[1]-self.mpos))))
                        self.text.update()
                if self.text.yview()[1] < 1:
                    self.scrollbar_in()
                else:
                    self.scrollbar_out()
                self.tk.deiconify()

    # hide window, when pointer leave the text widget
    def leave(self, event):
        # hide, if mode = background, and not open local menu
        if self.mode == 1 and self.local_on == 0:
            self.tk.withdraw()
            self.scrollbar_out()
        else:
            self.local_on = 0

    # call from menu
    def change_mode_event(self, event):
        self.change_mode()

    # change background <-> foreground mode
    def change_mode(self):
        self.mode = 1 - self.mode;
        self.tk.overrideredirect(self.mode)
        if self.mode == 0:
            self.localmenu.entryconfigure(4,label="Httr-zemmd")
            self.tk.config(menu=self.menu) # show menubar
            self.tk.withdraw()
            self.tk.deiconify()
            self.scrollbar_in()
            self.text.configure(bg="white")
        else:
            self.localmenu.entryconfigure(4,label="Eltr-zemmd".decode("iso-8859-2"))
            self.tk.config(menu="") # hide menubar
            self.scrollbar_out()
            self.text.configure(bg="light yellow")
            self.tk.withdraw()

    # show scrollbar
    def scrollbar_in(self):
            self.scrollbar.configure(width=15,borderwidth=1,elementborderwidth=1,highlightthickness=0)
    
    # hide scrollbar
    def scrollbar_out(self):
            self.scrollbar.configure(width=0,borderwidth=0,elementborderwidth=0,highlightthickness=0)

    def all_items(self):
        if self.prev_selection == "":
            self.clear()
            self.puts([self.empty_message2], 'idiom')
        else:
                dictext = self.dictionary.get_item(split(self.prev_selection)[0])
                diclen = len(dictext)
                self.clear()
                self.puts(self.prev_selection + '\n','item2')
                self.puts(dictext, 'item');
                dictext = self.dictionary.get_all(split(self.prev_selection)[0])
                if len(dictext) == 0:
                    if diclen == 0:
                        dictext = [self.empty_message]
                    else:
                        dictext = [self.empty_message3]                        
                self.puts(dictext, 'idiom');
        if self.mode != 2:
            self.set_windowsize()

    # show XQED informations    
    def about_xqed(self):
        self.clear()
        self.tk.overrideredirect(self.mode)
        self.text.insert(END,__doc__)
        self.set_windowsize()

    # show description of pronounciating signs
    def about_prono(self):
        try:
            s = open(self.qed_prono,'r').readlines()
        except:
            s = "A kiejtsi tmutat nem elrhet!"
#        if self.mode and not self.prev_prono:
        if not self.prev_prono:
            pron = ""
            dictext = self.dictionary.get_item(self.prev_selection)
            for i in dictext:
                k = split(i,'  ',1)
                kpro = re.split('[\[\]]',k[0])
                pron += kpro[1]
            for i in s:
                if (i[0] in self.hungarian_phonetic_conversion(pron)) and \
                    (i[0] in self.hungarian_phonetic_conversion("iIe&A0OUuV3@RTDNltw")):
                    wordlen = len(split(i," ")[0])
                    self.puts(i[0:wordlen],'idiom')
                    self.puts(i[wordlen:],'')
            self.prev_prono = 1
        else:
            self.clear()
            if self.mode != 2:
                self.tk.overrideredirect(self.mode)
            for i in s:
                    wordlen = len(split(i," ")[0])
                    self.puts(i[0:wordlen],'idiom')
                    self.puts(i[wordlen:],'')
        if self.mode != 2:
            self.set_windowsize()

    def about_program(self):
        self.clear()
        self.tk.overrideredirect(self.mode)
        self.puts("Az XQED angol-magyar sztrprogram", 'item')

# (A XQED program az X fellet azon tulajdonsgt \
# hasznlja ki, hogy az egr, vagy a billentyzet segtsgvel kijellt szveg automatikusan a \
# vglapra kerl. Az XQED program folyamatosan figyeli a vglap tartalmt, s ha vltozst \
# szlel, megprblja lefordtani azt. Ismtelt kijellsnl nincs vltozs.)

        self.puts("""\n
Az XQED program gyorsfordt, vagyis egr (vagy billentyzet) segtsgvel \
kijellt szavak gyors kisztrazst vgzi el. Jelljk ki a fordtani kvnt \
szt, vagy szavakat, s a fordts megjelenik ebben az ablakban. Sok \
programban (pldul a Mozilla bngszben) a szavak dupla kattintssal \
is kijellhetk.

Ugyanazon sz ismtelt kijellse

Egy sz ismtelt kijellst alapesetben nem veszi szre a program. \
Ilyenkor az a megolds, hogy az ismtelt kijellsbe egy-kt \
tovbbi karaktert, vagy szt is bevesznk. Esetleg kijellhetnk egy ms szt, s utna \
az elzt. Az XQED programot a -s kapcsolval indtva, a program a vglap trlsvel oldja \
meg az ismtelt kijells problmjt. Itt egyms utn tbbszr is kattinthatunk a szn, \
viszont mr csak az alkalmazson bell tudunk vglapot hasznlni a Beilleszts (vagy az annak \
megfelel) menpont segtsgvel.

Sztri informcik

A szavak s kifejezsek magyar jelentsei mellett amerikai s brit angol kiejtsi \
informcikat tartalmaz a sztr. A vonzatok, illetve kifejezsek megjelentshez \
az sszes kifejezs menpontot kell kivlasztani, ami legknnyebben a jobb egrgombbal \
elhvhat helyi menben lehetsges.

Eltr-zemmd

Kt mkdsi md kztt vlaszthatunk. \
Az eltr-zemmd esetn a sztri bejegyzsek cmsorral s mensorral \
rendelkez ablakban jelennek meg:\n\n""",'');
        if os.access(self.PIC_WINDOW, os.R_OK):
            self.picw = PhotoImage(file=self.PIC_WINDOW)
            self.text.image_create(END,image=self.picw);
        else:
            self.puts(self.PIC_ERROR,'idiom')
            
        self.puts("""\n
Hasznos lehet gy belltani az eltr-zemmdban lv ablakot, \
hogy mindig fell legyen. Ezt az ablak cmsornak menjben \
llthatjuk be (a legtbb ablakkezel tmogatja ezt a lehetsget).
        
Szavak bersa s kijellse

Az eltr-zemmd ablakba berhatk a fordtani kvnt kifejezsek. \
A bershoz kattintsunk az ablak egy res rszn, \
majd gpeljk be a szt, vagy szavakat. Egy sz \
dupla kattintssal is kijellhet (az egrgomb lenyomsval s \
az egr hzsval viszont nem). Tbb sz esetn csak a \
billentyzet hasznlhat: gpeljk be a fordtsra sznt szveget, \
s nyomjuk meg a ,,Shift-Home'' billentykombincit.

Httr-zemmd

A httr-zemmd kivlasztsval a program ablaka ideiglenesen eltnik. Amint kijellnk \
egy szt, vagy szvegrszt, az ablak s benne a fordts megjelenik a kijells felett:\n\n""",'');
        if os.access(self.PIC_BG, os.R_OK):
            self.picb = PhotoImage(file=self.PIC_BG)
            self.text.image_create(END,image=self.picb);
        else:
            self.puts(self.PIC_ERROR,'idiom')
        self.puts("""\n
Ez az ablak nem mozgathat, s ha az egrmutat elhagyja a terlett, \
magtl eltnik. Emiatt az ablak tartalmt knyelmesebb a kzps gomb \
lenyomsval, s az egr mozgatsval tovbbgrdteni, gyelve arra, hogy \
az egrmutat ne hagyja el az ablakot. Ha nagyon sok a kifejezs, \
rdemes lehet az eltr-zemmdba is visszavltani, s kinagytani az \
ablakot.

Helyi men -> Eltr-zemmd

Mivel a httr-zemmd ablaka mensorral nem rendelkezik, a helyi men segtsgvel \
trhetnk vissza az eltr-zemmdba:\n\n""",'');
        if os.access(self.PIC_LOCAL, os.R_OK):
            self.picl = PhotoImage(file=self.PIC_LOCAL)
            self.text.image_create(END,image=self.picl);
        else:
            self.puts(self.PIC_ERROR,'idiom')
        self.puts("""\n
Men -> sszes kifejezs
        
A helyi men els menpontjval az adott szhoz tartoz \
sszes kifejezst jelenthetjk meg. Dupla egrkattintssal \
kijellhetk s lefordthatk a kifejezsekben szerepl egyb \
szavak is. (Az eltr-zemmdban erre a billentyzet is hasznlhat.)

Men -> Felolvass

A Festival beszdszintetiztor megfelel teleptse esetn \
a kijellt szvegrsz felolvassra kerl.

Men -> Kiejtsi tmutat

A menpont els kivlasztsval az ablakban lthat \
fonmk kiejtsrl kaphatunk informcikat. A menpont \
ismtelt kivlasztsval az sszes fonma lersa megjelenik.\n\n""",'');
        
        self.puts("\nKifejezsek fordtsa", 'item')
        self.puts("""\n
Az XQED program tmogatja a kifejezsek fordtst. Minden kifejezs \
automatikusan kirsra kerl, amely a kt utoljra kijellt szt \
tartalmazza:\n\n""",'');
        if os.access(self.PIC_IDIOM, os.R_OK):
            self.pici = PhotoImage(file=self.PIC_IDIOM)
            self.text.image_create(END,image=self.pici);
        else:
            self.puts(self.PIC_ERROR,'idiom')
        self.puts("""\n
A kpen elszr a ,,thou'', majd az ,,art'' sz lett kijellve, gy \
kerlt kivlasztsra a piros sznnel megjelentett kifejezs.

Ha egyszerre tbb szt jellnk ki, akkor a kt szls sz alapjn \
kerlnek kivlasztsra a kifejezsek.\n\n""",'')
        self.set_windowsize()

    def about_magyar(self):
        self.clear()
        self.tk.overrideredirect(self.mode)
        self.puts("Magyarrl angolra fordts", 'item')
        self.puts("""\n
Az XQED a Vony Attila-fle angol-magyar sztr bvtett s rszben \
javtott vltozatt hasznlja s jelenleg csak az angolrl magyarra \
trtn fordtst tmogatja a grafikus fellet.

Ha magyar szavak, vagy kifejezsek angol nyelv megfeleljt szeretnnk \
megkeresni, akkor hasznljuk az angolul \
parancsot a parancssorban:\n\n""",'')
        self.puts("angolul macska\n", 'idiom')

        self.puts("""
A parancs bershoz egy terminlablakot kell megnyitnunk. Ha a lerst \
sznesben is ltni szeretnnk, a -c kapcsolt is adjuk meg:\n\n""",'')

        self.puts("angolul -c macska\n", 'idiom')

        self.puts("""
A parancssori fellet az X grafikus rendszer nlkl is knyelmesen \
hasznlhat angol kifejezsek fordtsra. A magyarul parancs \
lerst az xqed (vagy angolul, illetve magyarul) -h \
kapcsoljval krhetjk a parancssorban.\n\n""",'')
    
    # set local_on variable before showing local menu (see localmenu def.)
    def set_local_on(self):
        self.local_on = 1

    # show local menu
    def local_menu(self, event):
        t = self.tk.winfo_pointerxy()
        self.localmenu.tk_popup(t[0],t[1])

    def set_fontsize(self, num):
        self.text.configure(font=('helvetica', num))
        self.menu.configure(font=('helvetica', num))
        self.modemenu.configure(font=('helvetica', num))
        self.helpmenu.configure(font=('helvetica', num))
        self.localmenu.configure(font=('helvetica', num))
        self.fontmenu.configure(font=('helvetica', num))
        self.text.tag_configure('item', foreground="blue", font=('helvetica', num, 'bold'))
        self.text.tag_configure('item2', font=('helvetica', num + 2, 'bold'))
        self.text.tag_configure('prono', foreground="dark green", font=('helvetica', num, 'bold'))                
        self.text.tag_configure('idiom', foreground="red", font=('helvetica', num, 'bold'))

    def set_font8(self):
        self.set_fontsize(8)

    def set_font10(self):
        self.set_fontsize(10)

    def set_font12(self):
        self.set_fontsize(12)

    def set_font14(self):
        self.set_fontsize(14)

    def set_font18(self):
        self.set_fontsize(18)

    def set_font24(self):
        self.set_fontsize(24)

    def tts(self):
        try:
            f = open(self.festival + "/festival",'r')
            os.system("echo '" + self.prev_selection + "' | " + self.festival + "/festival --tts");
        except:
            self.clear()
            self.tk.overrideredirect(self.mode)
            self.text.insert(END,"A Festival beszdszintetiztor nem tallhat a " + \
                self.festival + " helyen.\n")

    def magyar(self, expressions, opt):
        opt_c = ""
        if self.options_c:
            opt_c = "-c"
        if opt == "q":
            os.system("egrep -i '" + r"([ ][ ]|, )" + expressions + r"($|[ ]|,)' " + \
                self.qed_dict + r" | gawk 'a[$0]!=1{a[$0]=1;print$0}'" + r" | xqed " + opt_c + " -f")
        else:
            os.system("gawk '/" + expressions + r"/{if(a[$0]!=1){a[$0]=1;print$0}}' " + \
                self.qed_dict + r" | xqed " + opt_c + " -f")

    def press_enter(self, event):
        self.text.tag_add("sel","insert linestart", "insert")
        self.puts(" ",'item')

    # exit from program
    def xqed_exit(self):
        sys.exit(0)

# Main program
def main():
    xqed = Xqed()
    xqed.tk.mainloop()

# Call main when run as script
if __name__ == '__main__':
    main()
